// -*- C++ -*-
/*!
 * @file  HumanDetection_Fusion.cpp
 * @brief HumanDetection_Fusion component
 * $Date$ 
 * @author Nemoto,Mizoguchi lab,Tokyo University of Science
 * @mail:j7507640@ed.noda.tus.ac.jp
 * $Id$
 */

#include "HumanDetection_Fusion.h"
#include "detection.h"
#include <math.h>
#define TARGET_H 1.20 
//#define CAMERA_H 1.27 //Enon使用時
#define CAMERA_H 0.90 //Segwey雲台用
// Module specification
// <rtc-template block="module_spec">
static const char* humandetection_fusion_spec[] =
  {
    "implementation_id", "HumanDetection_Fusion",
    "type_name",         "HumanDetecFusion",
    "description",       "HumanDetection_Fusion component",
    "version",           "1.0",
    "vendor",            "Nemoto,Mizoguchi Lab,Tokyo University of Science",
    "category",          "RobotModule",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.LRF_X_Pos", "0.0",
    "conf.default.LRF_Y_Pos", "0.0",
    "conf.default.LRF_Z_Pos", "0.0",
    "conf.default.Camera_X_Pos", "0.0",
    "conf.default.Camera_Y_Pos", "0.0",
    "conf.default.Camera_Z_Pos", "0.0",
    "conf.default.PanTilt_freedom", "2",

    ""
  };
// </rtc-template>

HumanDetection_Fusion::HumanDetection_Fusion(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_VisionPosIn("VisionPos", m_VisionPos),
    m_VisionAgreeIn("VisionAgree", m_VisionAgree),
    m_LRFPosIn("LRFPos", m_LRFPos),
    m_HumanPosOut("HumanPos", m_HumanPos),
    m_HumanPosXYOut("HumanPosXY", m_HumanPosXY),
    m_HumanPosXYZOut("HumanPosXYZ", m_HumanPosXYZ),
    m_ControlAngleOut("ControlAngle", m_ControlAngle),
    m_SetSonsorPosPortPort("SetSonsorPosPort"),
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
//  registerInPort("VisionPos", m_VisionPosIn);
//  registerInPort("VisionAgree", m_VisionAgreeIn);
//  registerInPort("LRFPos", m_LRFPosIn);
  
  // Set OutPort buffer
//  registerOutPort("HumanPos", m_HumanPosOut);
//  registerOutPort("HumanPosXY", m_HumanPosXYOut);
//  registerOutPort("HumanPosXYZ", m_HumanPosXYZOut);
//  registerOutPort("ControlAngle", m_ControlAngleOut);
  
  // Set service provider to Ports
//  m_SetSonsorPosPortPort.registerProvider("SetSonsorPosInfo", "SetSonsorPosInfo", m_SetSonsorPosInfo);
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
//  registerPort(m_SetSonsorPosPortPort);
  
  // </rtc-template>

}

HumanDetection_Fusion::~HumanDetection_Fusion()
{
}


RTC::ReturnCode_t HumanDetection_Fusion::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("VisionPos", m_VisionPosIn);
  addInPort("VisionAgree", m_VisionAgreeIn);
  addInPort("LRFPos", m_LRFPosIn);
  
  // Set OutPort buffer
  addOutPort("HumanPos", m_HumanPosOut);
  addOutPort("HumanPosXY", m_HumanPosXYOut);
  addOutPort("HumanPosXYZ", m_HumanPosXYZOut);
  addOutPort("ControlAngle", m_ControlAngleOut);
  
  // Set service provider to Ports
  m_SetSonsorPosPortPort.registerProvider("SetSonsorPosInfo", "SetSonsorPosInfo", m_SetSonsorPosInfo);
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  registerPort(m_SetSonsorPosPortPort);

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("LRF_X_Pos", m_LRF_X_Pos, "0.0");
  bindParameter("LRF_Y_Pos", m_LRF_Y_Pos, "0.0");
  bindParameter("LRF_Z_Pos", m_LRF_Z_Pos, "0.0");
  bindParameter("Camera_X_Pos", m_Camera_X_Pos, "0.0");
  bindParameter("Camera_Y_Pos", m_Camera_Y_Pos, "0.0");
  bindParameter("Camera_Z_Pos", m_Camera_Z_Pos, "0.0");
  bindParameter("PanTilt_freedom", m_PanTilt_freedom, "2");
  
  // </rtc-template>
  return RTC::RTC_OK;
}




RTC::ReturnCode_t HumanDetection_Fusion::onFinalize()
{
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t HumanDetection_Fusion::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t HumanDetection_Fusion::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t HumanDetection_Fusion::onActivated(RTC::UniqueId ec_id)
{

	bool *port_connect_check;
	  PortServiceList* portlist;
    portlist = this->get_ports();
    port_connect_check=new bool [portlist->length()];

    for (CORBA::ULong i(0), n(portlist->length()); i < n; ++i)
	{
	    PortService_ptr port;
	    port = (*portlist)[i];
	    cout << "PortNum"<<i<<":"<<port->get_port_profile()->name << "=";
	    if(port->get_port_profile()->connector_profiles.length()!=0){
		port_connect_check[i]=true;
		cout <<"Connect"<<endl;
	    }
	    else if(port->get_port_profile()->connector_profiles.length()==0){
		port_connect_check[i]=false;
		cout <<"DisConnect"<<endl;
	    }
	}
    cout <<"-----------------------"<<endl;

  		    PortService_ptr port = (*portlist)[7];//This port is SensorPos Service Port
    	if(port->get_port_profile()->connector_profiles.length()!=0){
			cout<<"SensorPos Service Port connect"<<endl;
			LRF_Set_X=m_SetSonsorPosInfo.LRF_Set_X;
			LRF_Set_Y=m_SetSonsorPosInfo.LRF_Set_Y;
			LRF_Set_Z=m_SetSonsorPosInfo.LRF_Set_Z;
			Camera_Set_X=m_SetSonsorPosInfo.Camera_Set_X;
			Camera_Set_Y=m_SetSonsorPosInfo.Camera_Set_Y;
			Camera_Set_Z=m_SetSonsorPosInfo.Camera_Set_Z;
		}
		 else if(port->get_port_profile()->connector_profiles.length()==0){
			cout<<"SensorPos Service Port disconnect, Use configure parameter"<<endl;
			LRF_Set_X=m_LRF_X_Pos;
			LRF_Set_Y=m_LRF_Y_Pos;
			LRF_Set_Z=m_LRF_Z_Pos;
			Camera_Set_X=m_Camera_X_Pos;
			Camera_Set_Y=m_Camera_Y_Pos;
		Camera_Set_Z=m_Camera_Z_Pos;
		}

    panOut=0;
    tiltOut=0;
    vision_x=0,vision_y=0;
    humanPos_x=0,humanPos_y=0;
    lrf_x=0,lrf_y=0;
    vision_pan=0,vision_tilt=0;
    
    m_ControlAngle.pan=0;
    m_ControlAngle.tilt=0;
    m_ControlAngleOut.write();

  return RTC::RTC_OK;
}



RTC::ReturnCode_t HumanDetection_Fusion::onDeactivated(RTC::UniqueId ec_id)
{
    m_ControlAngle.pan=0;
    m_ControlAngle.tilt=0;
    m_ControlAngleOut.write();

  return RTC::RTC_OK;
}



RTC::ReturnCode_t HumanDetection_Fusion::onExecute(RTC::UniqueId ec_id)
{

    //from stereo
    if (m_VisionPosIn.isNew()) {
	m_VisionPosIn.read();
	vision_x=m_VisionPos.x+Camera_Set_X;
	vision_y=m_VisionPos.y+Camera_Set_Y;
	vision_z=m_VisionPos.z+Camera_Set_Z;
    }

    //from stereo
    if (m_VisionAgreeIn.isNew()) {
	m_VisionAgreeIn.read();
	vision_pan=m_VisionAgree.pan;
	vision_tilt=m_VisionAgree.tilt;
    }

    //from lrf
    if (m_LRFPosIn.isNew()) {
	m_LRFPosIn.read();
	lrf_x=m_LRFPos.x+LRF_Set_X;
	lrf_y=m_LRFPos.y+LRF_Set_Y;
	lrf_z=0;
    }

    if((vision_x!=0)&&(vision_y!=0)){
		humanPos_y = vision_y;
		humanPos_x = vision_x;
		humanPos_z = vision_z;
		panOut=vision_pan;
		tiltOut=vision_tilt;
    }


    if((lrf_x!=0)&&(lrf_y!=0)){

	Cal_XYPan(lrf_x,lrf_y,panOut);
	double length=sqrt(lrf_x*lrf_x+lrf_y*lrf_y);
        //tiltOut=0.1309*length*length-0.6545*length+0.7854;
        if(Camera_Set_Z!=0)tiltOut=atan2((double)((double)TARGET_H-(double)Camera_Set_Z),length);
        if(Camera_Set_Z==0)tiltOut=atan2((double)((double)TARGET_H-(double)CAMERA_H),length);
	if(tiltOut>20.0*PI/180.0) tiltOut=20.0*PI/180.0;
        if(tiltOut<-20.0*PI/180.0) tiltOut=-20.0*PI/180.0;
        //printf("%lf,deg=%lf\n",length,tiltOut);
	//if(length>2.0) tiltOut=0;
		humanPos_z = lrf_z;
		humanPos_y = lrf_y;
		humanPos_x = lrf_x;
    }

    if((vision_x==0)&&(vision_y==0)){
	    if((lrf_x==0)&&(lrf_y==0)){
			humanPos_z=0;
			humanPos_x=0;
			humanPos_y=0;
		}
	}

	if(m_PanTilt_freedom==0) {
		panOut=0;
		tiltOut=0;
		}
	else if(m_PanTilt_freedom==1) {
		panOut=0;
	}



//	cout<<"Vision position x:"<<vision_x<<"  "<<"y:"<<vision_y<<"|"<<"LRF position x:"<<lrf_x<<"  "<<"y:"<<lrf_y<<"|"<<"target x:"<<humanPos_x<<"  "<<"y:"<<humanPos_y<<endl;

    //to run mechanism
    m_HumanPos.x=humanPos_x;
    m_HumanPos.y=humanPos_y;
    m_HumanPosOut.write();

    m_HumanPosXY.x=humanPos_x;
    m_HumanPosXY.y=humanPos_y;
    m_HumanPosXYOut.write();

    m_HumanPosXYZ.x=humanPos_x;
    m_HumanPosXYZ.y=humanPos_y;
    m_HumanPosXYZ.z=humanPos_z;
    m_HumanPosXYZOut.write();

    m_ControlAngle.pan=panOut;
    m_ControlAngle.tilt=tiltOut;
    m_ControlAngleOut.write();

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t HumanDetection_Fusion::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t HumanDetection_Fusion::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t HumanDetection_Fusion::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t HumanDetection_Fusion::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t HumanDetection_Fusion::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void HumanDetection_FusionInit(RTC::Manager* manager)
  {
    RTC::Properties profile(humandetection_fusion_spec);
    manager->registerFactory(profile,
                             RTC::Create<HumanDetection_Fusion>,
                             RTC::Delete<HumanDetection_Fusion>);
  }
  
};


