// -*- C++ -*-
/*!
 * @file  HumanDetection_Fusion.h
 * @brief HumanDetection_Fusion component
 * @date 
 * @author Nemoto,Mizoguchi lab,Tokyo University of Science
 * @mail:j7507640@ed.noda.tus.ac.jp
 * $Id$
 */

#ifndef HUMANDETECTION_FUSION_H
#define HUMANDETECTION_FUSION_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">
#include "SetSonsorPosInfoSVC_impl.h"

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">
#include "MyDataTypeStub.h"

// </rtc-template>

using namespace RTC;

#include <iostream>
#include "type.h"
using namespace std;

class HumanDetection_Fusion
  : public RTC::DataFlowComponentBase
{
 public:
  HumanDetection_Fusion(RTC::Manager* manager);
  ~HumanDetection_Fusion();

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry() 
 virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
 virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  // The activated action (Active state entry action)
  // former rtc_active_entry()
virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // Configuration variable declaration
  // <rtc-template block="config_declare">
  double m_LRF_X_Pos;
  double m_LRF_Y_Pos;
  double m_LRF_Z_Pos;
  double m_Camera_X_Pos;
  double m_Camera_Y_Pos;
  double m_Camera_Z_Pos;
  int m_PanTilt_freedom;
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedXYZDouble m_VisionPos;
  InPort<TimedXYZDouble> m_VisionPosIn;
  TimedPanTilt m_VisionAgree;
  InPort<TimedPanTilt> m_VisionAgreeIn;
  TimedXYDouble m_LRFPos;
  InPort<TimedXYDouble> m_LRFPosIn;
  
  // </rtc-template>

  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedOdometry m_HumanPos;
  OutPort<TimedOdometry> m_HumanPosOut;
  TimedXYDouble m_HumanPosXY;
  OutPort<TimedXYDouble> m_HumanPosXYOut;
  TimedXYZDouble m_HumanPosXYZ;
  OutPort<TimedXYZDouble> m_HumanPosXYZOut;
  TimedPanTilt m_ControlAngle;
  OutPort<TimedPanTilt> m_ControlAngleOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  RTC::CorbaPort m_SetSonsorPosPortPort;
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  SetSonsorPosInfoSVC_impl m_SetSonsorPosInfo;
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  int dummy;

 double panOut;
 double tiltOut;
 double vision_x,vision_y,vision_z;
 double humanPos_x,humanPos_y,humanPos_z;
 double lrf_x,lrf_y,lrf_z;
 double vision_pan,vision_tilt;

double LRF_Set_X;
double LRF_Set_Y;
double LRF_Set_Z;
double Camera_Set_X;
double Camera_Set_Y;
double Camera_Set_Z;

};


extern "C"
{
  void HumanDetection_FusionInit(RTC::Manager* manager);
};

#endif // HUMANDETECTION_FUSION_H
