// -*- C++ -*-
/*!
 * @file  LocalPathPlanning.cpp
 * @brief LocalPathPlanning component
 * $Date$ 
 * @author Nemoto,Mizoguchi lab,Tokyo University of Science
 * @mail:j7507640@ed.noda.tus.ac.jp
 * $Id$ 
 */
#include "LocalPathPlanning.h"

// Module specification
// <rtc-template block="module_spec">
static const char* localpathplanning_spec[] =
  {
    "implementation_id", "LocalPathPlanning",
    "type_name",         "LocalPathPlanning",
    "description",       "LocalPathPlanning component",
    "version",           "1.0",
    "vendor",            "Nemoto,Mizoguchi Lab,Tokyo University of Science",
    "category",          "RobotModule",
    "activity_type",     "SPORADIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.max_mps", "1.0",
    "conf.default.max_rps", "1.5",
    "conf.default.min_mps", "0.0",
    "conf.default.min_rps", "-1.5",
    "conf.default.accelerate_mps", "0.8",
    "conf.default.accelerate_rps", "1.5",
    ""
  };
// </rtc-template>

LocalPathPlanning::LocalPathPlanning(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_InputCtrlIn("InputCtrl", m_InputCtrl),
    m_TargetPosIn("TargetPos", m_TargetPos),
    m_OdometryIn("Odometry", m_Odometry),
    m_VelocityOut("Velocity", m_Velocity),
    m_RunControlPORTPort("RunControlPORT")

    // </rtc-template>
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("InputCtrl", m_InputCtrlIn);
  registerInPort("TargetPos", m_TargetPosIn);
  registerInPort("Odometry", m_OdometryIn);

  // Set OutPort buffer
  registerOutPort("Velocity", m_VelocityOut);

  // Set service provider to Ports
  m_RunControlPORTPort.registerProvider("RunControlInfo", "RunControlInfo", m_RunControlInfo);

  // Set service consumers to Ports

  // Set CORBA Service Ports
  registerPort(m_RunControlPORTPort);

  // </rtc-template>

}

LocalPathPlanning::~LocalPathPlanning()
{
}


RTC::ReturnCode_t LocalPathPlanning::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("max_mps", m_max_mps, "1.0");
  bindParameter("max_rps", m_max_rps, "1.5");
  bindParameter("min_mps", m_min_mps, "0.0");
  bindParameter("min_rps", m_min_rps, "-1.5");
  bindParameter("accelerate_mps", m_accelerate_mps, "0.8");
  bindParameter("accelerate_rps", m_accelerate_rps, "1.5");

  // </rtc-template>
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t LocalPathPlanning::onFinalize()
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t LocalPathPlanning::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t LocalPathPlanning::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t LocalPathPlanning::onActivated(RTC::UniqueId ec_id)
{
    PortServiceList* portlist;
    portlist = this->get_ports();

	port_connect_check=new bool [portlist->length()];
	cout<<endl;
    for (CORBA::ULong i(0), n(portlist->length()); i < n; ++i)
	{
	    PortService_ptr port;
	    port = (*portlist)[i];
	    cout << "PortNum"<<i<<":"<<port->get_port_profile()->name << "=";
	    if(port->get_port_profile()->connector_profiles.length()!=0){
		port_connect_check[i]=true;
		cout <<"Connect"<<endl;
	    }
	    else if(port->get_port_profile()->connector_profiles.length()==0){
		port_connect_check[i]=false;
		cout <<"DisConnect"<<endl;
	    }
	}
	cout<<endl;

    robot_x=0,robot_y=0;
    speed=0,turn=0;
    goal_x=0,goal_y=0;
    now_t=0,old_t=0;

    old_speed=0;
    old_turn=0;

    m_Velocity.v=speed;
    m_Velocity.w=turn;
    m_VelocityOut.write();
    

	m_RunControlInfo.MpsAcc=m_accelerate_mps;
	m_RunControlInfo.RpsAcc=m_accelerate_rps;
	m_RunControlInfo.MpsMax=m_max_mps;
	m_RunControlInfo.RpsMax=m_max_rps;
	m_RunControlInfo.MpsMin=m_min_mps;
	m_RunControlInfo.RpsMin=m_min_rps;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t LocalPathPlanning::onDeactivated(RTC::UniqueId ec_id)
{
    speed=0,turn=0;
    m_Velocity.v=speed;
    m_Velocity.w=turn;
    m_VelocityOut.write();

  return RTC::RTC_OK;
}


RTC::ReturnCode_t LocalPathPlanning::onExecute(RTC::UniqueId ec_id)
{
    static double robot_theta;
    usleep(10*1000);

	if(port_connect_check[4]==false){
		m_RunControlInfo.MpsAcc=m_accelerate_mps;
		m_RunControlInfo.RpsAcc=m_accelerate_rps;
		m_RunControlInfo.MpsMax=m_max_mps;
		m_RunControlInfo.RpsMax=m_max_rps;
		m_RunControlInfo.MpsMin=m_min_mps;
		m_RunControlInfo.RpsMin=m_min_rps;
	}
    
    if (m_OdometryIn.isNew()) {
	m_OdometryIn.read();
	robot_x=m_Odometry.x;
	robot_y=m_Odometry.y;
	robot_theta=m_Odometry.theta;
    }
        
    if (m_TargetPosIn.isNew()) {
	m_TargetPosIn.read();
	goal_x=m_TargetPos.x;
	goal_y=m_TargetPos.y;
    if((goal_x!=0)&&(goal_y!=0)){
       	follow_velocity(goal_x,goal_y,speed,turn);
    }
    else{
		speed=0.0;
		turn=0.0;	
    }

    }
    
 /*   if (m_InputCtrlIn.isNew()) {
		m_InputCtrlIn.read();
	//	if((m_InputCtrl.v!=0)&&(m_InputCtrl.w!=0)){
			speed=m_InputCtrl.v;
			turn=m_InputCtrl.w;
	//}


    }*/
        
    if(speed>m_RunControlInfo.MpsMax) speed=m_RunControlInfo.MpsMax;
    if(speed<m_RunControlInfo.MpsMin) speed=m_RunControlInfo.MpsMin;
    if(turn>m_RunControlInfo.RpsMax) turn=m_RunControlInfo.RpsMax;
    if(turn<m_RunControlInfo.RpsMin) turn=m_RunControlInfo.RpsMin;

    now_t=timer_ms();
    if((now_t!=0)&&(old_t!=0)&&((now_t-old_t)>0)){

	mps_variation=(now_t-old_t)/1000.0*m_RunControlInfo.MpsAcc;
	rps_variation=(now_t-old_t)/1000.0*m_RunControlInfo.RpsAcc;

	if(fabs(speed-old_speed)>mps_variation){
	    speed=old_speed+(speed-old_speed)/fabs(speed-old_speed)*mps_variation;
	}
	if(fabs(turn-old_turn)>rps_variation){
	    turn=old_turn+(turn-old_turn)/fabs(turn-old_turn)*rps_variation;
	}

	cout<<speed<<" "<<turn<<":"<<"/"<<mps_variation<<rps_variation<<"/"<<m_RunControlInfo.MpsAcc<<":"<<m_RunControlInfo.RpsAcc<<endl;

    }

    
    old_t= timer_ms();
    old_speed=speed;
    old_turn=turn;
    m_Velocity.v=speed;
    m_Velocity.w=turn;
    m_VelocityOut.write();
	

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t LocalPathPlanning::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t LocalPathPlanning::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t LocalPathPlanning::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t LocalPathPlanning::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t LocalPathPlanning::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


extern "C"
{
 
  void LocalPathPlanningInit(RTC::Manager* manager)
  {
    RTC::Properties profile(localpathplanning_spec);
    manager->registerFactory(profile,
                             RTC::Create<LocalPathPlanning>,
                             RTC::Delete<LocalPathPlanning>);
  }
  
};



