// -*- C++ -*-
/*!
 * @file  LocalPathPlanning.h 
* @brief LocalPathPlanning component
 * @date  $Date$ 
 * @author Nemoto,Mizoguchi lab,Tokyo University of Science
 * @mail:j7507640@ed.noda.tus.ac.jp
 * $Id$ 
 */
#ifndef LOCALPATHPLANNING_H
#define LOCALPATHPLANNING_H

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>

// Service implementation headers
// <rtc-template block="service_impl_h">
#include "MyDataTypeSVC_impl.h"

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

using namespace RTC;

#include <iostream>
#include "timer.h"
#include "pathplan.h"
#include <math.h>
using namespace std;

class LocalPathPlanning  : public RTC::DataFlowComponentBase
{
 public:
  LocalPathPlanning(RTC::Manager* manager);
  ~LocalPathPlanning();

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry() 
 virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  // virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  // The activated action (Active state entry action)
  // former rtc_active_entry()
virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // Configuration variable declaration
  // <rtc-template block="config_declare">
  double m_max_mps;
  double m_max_rps;
  double m_min_mps;
  double m_min_rps;
  double m_accelerate_mps;
  double m_accelerate_rps;
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedVelocity m_InputCtrl;
  InPort<TimedVelocity> m_InputCtrlIn;
  TimedXYDouble m_TargetPos;
  InPort<TimedXYDouble> m_TargetPosIn;
  TimedOdometry m_Odometry;
  InPort<TimedOdometry> m_OdometryIn;

  // </rtc-template>

  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedVelocity m_Velocity;
  OutPort<TimedVelocity> m_VelocityOut;

  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  RTC::CorbaPort m_RunControlPORTPort;

  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  RunControlInfoSVC_impl m_RunControlInfo;

  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">

  // </rtc-template>

 private:

    double robot_x,robot_y;
    double speed,turn;
    double goal_x,goal_y;
    double now_t,old_t;
    double old_speed,old_turn;
    double mps_variation;
    double rps_variation;
	bool *port_connect_check;    
};


extern "C"
{
  void LocalPathPlanningInit(RTC::Manager* manager);
};

#endif // LOCALPATHPLANNING_H

